from typing import Any


class PatternMeta:
    def __init__(self, id: int, name: str, frames: int,
                 pixels: int, flags: int, fps: int):
        self.id: int = id
        self.name: str = name
        self.frames: int = frames
        self.pixels: int = pixels
        self.flags: int = flags
        self.fps: int = fps

    def to_json(self) -> Any:
        return {
            "id": self.id, "name": self.name, "frames": self.frames,
            "pixels": self.pixels, "flags": self.flags, "fps": self.fps
        }

    @classmethod
    def from_json(cls, json: Any):
        return cls(
            json["id"], json["name"], json["frames"],
            json["pixels"], json["flags"], json["fps"]
        )


class PatternBuilder:
    def __init__(self, name: str, fps=1):
        self.pixels: list[list[int]] = []
        self.name: str = name
        self.fps: int = fps
        self.frame_count: int = 1
        self.pixel_count: int = 0
        self.frame_pixels: int = 0

    def add_pixel(self, r: int, g: int, b: int):
        if self.frame_count == 1:
            self.pixel_count += 1

        self.frame_pixels += 1
        self.pixels += [r, g, b]

    def next_frame(self):
        self.frame_count += 1
        self.frame_pixels = 0

    def is_valid(self):
        self.frame_pixels == self.pixel_count

    def to_binary_string(self) -> str:
        return ''.join([chr(item) for item in self.pixels])
